(* Copyright (c) 2003 Michal Moskal <malekith@pld-linux.org>
   Licensed under the Academic Free License version 2.0 *)
type ctx =
  {
    filename : string;
    channel : in_channel;
    include_chain : bool;
    mutable line : int;
  }

let err_cnt = ref 0
let ctx_stack = Stack.create ()
let ctx () = Stack.top ctx_stack
  
let err msg =
  incr err_cnt;
  Printf.eprintf "%s:%d: %s\n" (ctx ()).filename (ctx ()).line msg

let include_file name =
  try
    let ch = open_in name in
    let c =
      {
        filename = name;
        channel = ch; 
        line = 0; 
        include_chain = true
      }
    in
    Stack.push c ctx_stack;
  with 
    | Sys_error m -> err m

let in_context f name =
  let process name ch =
    let c =
      {
        filename = name;
        channel = ch; 
        line = 0; 
        include_chain = false
      }
    in
    Stack.push c ctx_stack;
    f ();
    ignore (Stack.pop ctx_stack)
  in
  if name = "-" then
    process "stdin" stdin
  else
    try
      let f = open_in name in
      process name f;
      close_in f
    with 
      | Sys_error m as e ->
        if Stack.is_empty ctx_stack then
          (prerr_endline m; exit 1)
        else err m

let rec fold_lines f ini =
  let rec line () =
    try
      (ctx ()).line <- (ctx ()).line + 1;
      Some (input_line (ctx ()).channel)
    with End_of_file -> 
      if (ctx ()).include_chain then begin
        close_in (ctx ()).channel;
        ignore (Stack.pop ctx_stack);
        line ()
      end else None
  in
  match line () with
  | Some line -> fold_lines f (f ini line)
  | None -> ini

let fold_logical_lines f ini =
  let concat acc = String.concat "\n" (List.rev acc) in
  let loop (acc, ctx) line =
    let len = String.length line - 1 in
    if len >= 0 && line.[len] = '\\' then
      (String.sub line 0 len :: acc, ctx)
    else
      if acc <> [] then ([], f ctx (concat (line :: acc)))
      else ([], f ctx line)
  in 
  match fold_lines loop ([], ini) with
  | ([], ctx) -> ctx
  | (acc, ctx) -> f ctx (concat acc)
