(* Copyright (c) 2003 Michal Moskal <malekith@pld-linux.org>
   Licensed under the Academic Free License version 2.0 *)
type macro =
  | Simple_macro of string
  | Param_macro of string   (* () *)
  | Simple_callback of (unit -> string)
  | Param_callback of (string -> string)
  
let globals = Hashtbl.create 100
let locals = ref []

let is_global_ctx () = !locals = []

let add ?(local = false) n v =
  match !locals with
  | x :: _ when local -> Hashtbl.replace x n v
  | _ -> Hashtbl.replace globals n v

let add_macro ?(local = false) name value =
  if My_String.ends_with name "()" then
    add ~local:local 
        (String.sub name 0 (String.length name - 2))
        (Param_macro (My_String.strip value))
  else
    add ~local:local name (Simple_macro (My_String.strip value))

let init_macros () =
  let include_file s =
    Generic_parser.include_file (My_String.strip s);
    ""
  in
  let cb local s =
    let (x, xs) = My_String.first_word (My_String.strip_left s) in
    if x <> "" then add_macro ~local:local x xs else ();
    ""
  in
  add_macro "_target" !Flags.target;
  add_macro "_target_os" !Flags.target_os;
  add_macro "_target_cpu" !Flags.target_cpu;
  add "define" (Param_callback (cb true));
  add "global" (Param_callback (cb false));
  add "include" (Param_callback include_file)

let get n = 
  let rec loop = function
    | x :: xs when Hashtbl.mem x n -> Hashtbl.find x n
    | _ :: xs -> loop xs
    | [] -> Hashtbl.find globals n
  in 
  loop !locals

let defined n = 
  try ignore (get n); true with Not_found -> false

let push_ctx args =
  let h = Hashtbl.create (List.length args) in
  let rec add no = function
    | x :: xs -> 
      Hashtbl.replace h (string_of_int no) (Simple_macro x); 
      add (no + 1) xs
    | [] -> ()
  in add 1 args;
  locals := h :: !locals

let pop_ctx () = locals := List.tl !locals
