(* Copyright (c) 2003 Michal Moskal <malekith@pld-linux.org>
   Licensed under the Academic Free License version 2.0 *)
let xf = Printf.sprintf

let progname = Sys.argv.(0)

let die msg =
  Printf.eprintf "%s: %s\n" progname msg;
  Printf.eprintf "%s: use %s --help to get more info\n" progname progname;
  exit 1
  
let spec = ref ""
let eval_str = ref None

let args = [
  "--shell", Arg.Set Flags.shell, "allow %(...) expansion";
  "--no-shell", Arg.Clear Flags.shell, "disallow %(...) expansion";
  "--with", Arg.String (fun s -> Macro.add_macro ("_with_" ^ s) ("--with-" ^ s)), 
            "define _with_ARG";
  "--without", Arg.String (fun s -> Macro.add_macro ("_without_" ^ s) ("--without-" ^ s)), 
            "define _without_ARG";
  "--target", Arg.String (fun s -> Flags.target := s), "set %{_target}";
  "--all", Arg.Set Flags.dump_all, "dump all sections; not only preamble";
  "--spec", Arg.String (fun s -> if !spec = "" then spec := s 
                                 else die "more then one spec passed"),
           "specify .spec file to parse";
  "--eval", Arg.String (fun s -> match !eval_str with 
                                 | None -> eval_str := Some s
                                 | Some _ -> die "more then one --eval passed"),
           "specify macros to eval";
  (* TODO: --macros *)
]

let usage = "Preprocess RPM spec files"

let sanitize_target () =
  let target =
    if !Flags.target = "" then
      Shell.backticks "rpm --eval '%{_target}'"
    else !Flags.target
  in
  let gr n = Str.matched_group n target in
  My_String.re_match target [
    ("^\\([^-]+\\)-\\([^-]+-\\)?\\([^-]+\\)\\(-gnu\\)?$", 
     fun () -> Flags.target_cpu := gr 1;
               Flags.target_os := gr 3);
    (".*", fun () -> die (xf "cannot parse target '%s'" target))
  ];
  Flags.target := !Flags.target_cpu ^ "-" ^ !Flags.target_os 

let main () =
  Arg.parse args (fun _ -> die "non-option arguments passed") usage;
  sanitize_target ();
  match !spec, !eval_str with
  | "", None -> die "no --spec and no --eval passed"
  | spec, None ->
    Macro.init_macros ();
    Macros_parser.process_all_macro_files ();
    Spec_parser.process_spec spec
  | spec, Some evl ->
    Macro.init_macros ();
    Flags.no_dump := true;
    Macros_parser.process_all_macro_files ();
    if spec = "" then ()
    else Spec_parser.process_spec spec;
    print_endline (Expander.expand 0 evl)

let () = main ()
