(* Copyright (c) 2003 Michal Moskal <malekith@pld-linux.org>
   Licensed under the Academic Free License version 2.0 *)
let starts_with s1 s2 =
  let l1 = String.length s1 in
  let l2 = String.length s2 in
  (l2 <= l1) && String.sub s1 0 l2 = s2

let ends_with s1 s2 =
  let l1 = String.length s1 in
  let l2 = String.length s2 in
  (l2 <= l1) && String.sub s1 (l1 - l2) l2 = s2

let strip_left s =
  let len = String.length s in
  let rec loop idx =
    if idx >= len then ""
    else
      match s.[idx] with
      | ' ' | '\t' | '\n' -> loop (idx + 1)
      | _ -> String.sub s idx (len - idx)
  in loop 0

let strip_right s =
  let rec loop idx =
    if idx < 0 then ""
    else
      match s.[idx] with
      | ' ' | '\t' | '\n' -> loop (idx - 1)
      | _ -> String.sub s 0 (idx + 1)
  in loop (String.length s - 1)

let strip s =
  strip_right (strip_left s)

let chop_prefix n s =
  String.sub s n (String.length s - n)
  
let first_word s =
  let len = String.length s in
  let rec loop idx =
    if idx >= len then (s, "")
    else
      match s.[idx] with
      | ' ' | '\t' | '\n' ->
        (String.sub s 0 idx, strip_left (chop_prefix idx s))
      | _ -> loop (idx + 1)
  in loop 0

let re_match s lst =
  let rec loop = function
    | (re, f) :: xs ->
      if Str.string_match (Str.regexp re) s 0 then
        f ()
      else
        loop xs
    | [] -> raise (Failure "re_match")
  in loop lst

let split s =
  let rec loop acc s =
    match first_word s with
    | ("", _) -> List.rev acc
    | (x, xs) -> loop (x :: acc) xs
  in loop [] (strip s)
